### R code from vignette source 'lifecontingenciesVignettes.Rnw'

###################################################
### code chunk number 1: lifecontingenciesVignettes.Rnw:76-79
###################################################
	options(prompt = "R> ", continue = "+  ", width = 70, useFancyQuotes = FALSE)
	library(lifecontingencies)
	source("calculations1.R")


###################################################
### code chunk number 2: lifecontingenciesVignettes.Rnw:83-84 (eval = FALSE)
###################################################
## 	source("calculations2.R")


###################################################
### code chunk number 3: load
###################################################
library(lifecontingencies)


###################################################
### code chunk number 4: show classes
###################################################
#definition of lifetable
showClass("lifetable")


###################################################
### code chunk number 5: show actuarial
###################################################
showClass("actuarialtable")


###################################################
### code chunk number 6: interest rates functions
###################################################
#an APR of 3% is equal to a 
real2Nominal(0.03,12)
#of nominal interest rate while
#6% annual  nominal interest rate is the same of 
nominal2Real(0.06,12)

#APR
#4% per year compounded quarterly is
nominal2Real(0.04,4)
#4% effective interest rate corresponds to 
real2Nominal(0.04,4)*100
#nominal interest rate (in 100s) compounded quarterly


###################################################
### code chunk number 7: discount rates function
###################################################
#a nominal rate of discount of 4% payable quarterly is equal to a 
real2Nominal(i=0.04,m=12,type="discount")


###################################################
### code chunk number 8: present value
###################################################
#suppose an investment requires and grants following cash flows
capitals=c(-1000,200,500,700)
#at time (vector) t.
times=c(-2,-1,4,7)
#the preset value of the investment is 
presentValue(cashFlows=capitals, timeIds=times, 
             interestRates=0.03)
#assuming 3% interest rate

  #while if interest rates were time - varying 
#e.g. 0.04 0.02 0.03 0.057
presentValue(cashFlows=capitals, timeIds=times, 
             interestRates=c( 0.04, 0.02, 0.03, 0.057))
#and if the last cash flow is uncertain, as we assume a 
#receiving probability of 50% 
presentValue(cashFlows=capitals, timeIds=times, 
interestRates=c( 0.04, 0.02, 0.03, 0.057), 
             probabilities=c(1,1,1,0.5))


###################################################
### code chunk number 9: annuities
###################################################
#PV annuity immediate 100$ each year 5 years @9%
100*annuity(i=0.09,n=5)
#while the corresponding future values is
100*accumulatedValue(i=0.09,n=5)


###################################################
### code chunk number 10: savings account projection
###################################################

#A man wants to save 100,000 to pay for the education
#of his son in 10 years time. An education fund requires the investors to
#deposit equal instalments annually at the end of each year. If interest of
#0.075 is paid, how much does the man need to save each year (R) in order to
#meet his target?
100000/accumulatedValue(i=0.075,n=10)


###################################################
### code chunk number 11: fractional annuities
###################################################
#Find the present value of an annuity-immediate of
#100 per quarter for 4 years, if interest is compounded semiannually at
#the nominal rate of 6%.
#the APR is
APR=nominal2Real(0.06,2)
100*4*annuity(i=APR,n=4,m=4)


###################################################
### code chunk number 12: increasing and decreasing
###################################################
#An increasing n-payment annuity-due shows payments of 1, 2,
#... , n
#at time 0, 1, ... , 
#n - 1 . At interest rate of 
#0.03 and n=10, its present value of the annuity is
increasingAnnuity(i=0.03, n=10,type="due")
#while the present value of a decreasing 
#annuity due of 10, 9,...,1 
#from time 1 to time 10 is
decreasingAnnuity(i=0.03, n=10,type="immediate")


###################################################
### code chunk number 13: geometrically increasing annuity
###################################################
#assume each year the annuity increases its value by 3%
#while the interest rate is 4%
#first determine the effective interest rate
ieff=(1+0.04)/(1+0.03)-1
#assume the annuity lasts 10 years
annuity(i=ieff,n=10)


###################################################
### code chunk number 14: capital amortization
###################################################
capital=100000
interest=0.05 #assume 5% effective annual interest
payments_per_year=2 #payments per year
rate_per_period=(1+interest)^(1/payments_per_year)-1
years=5 #five years length of the loan
installment=1/payments_per_year*capital/annuity(i=interest, n=years,m=payments_per_year)
installment	
#compute the balance due at the begin of period
balance_due=numeric(years*payments_per_year)
balance_due[1]=capital*(1+rate_per_period)-installment
for(i in 2:length(balance_due))
{
	balance_due[i]=balance_due[i-1]*(1+rate_per_period)-installment
	cat("Payment ",i, " balance due:",round(balance_due[i]),"\n")
}


###################################################
### code chunk number 15: bond pricing
###################################################
bond<-function(faceValue, couponRate, couponsPerYear, yield,maturity)
{
	out=NULL
	numberOfCF=maturity*couponsPerYear #determine the number of CF
	CFs=numeric(numberOfCF)
	payments=couponRate*faceValue/couponsPerYear #determine the coupon sum
	cf=payments*rep(1,numberOfCF)
	cf[numberOfCF]=faceValue+payments #set the last payment amount
	times=seq.int(from=1/couponsPerYear, to=maturity, by=maturity/numberOfCF)
	out=presentValue(cashFlows=cf, interestRates=yield, timeIds=times)
	return(out)
}
#bond coupon rate 6%, two coupons per year, face value 1000, yield 5%, three years to maturity
bond(1000,0.06,2,0.05,3)
#bond coupon rate 3%, one coupons per year, face value 1000, yield 3%, three years to maturity
bond(1000,0.06,1,0.06,3)


###################################################
### code chunk number 16: create a lifecontingencies object
###################################################
x_example=seq(from=0,to=9, by=1)
lx_example=c(1000,950,850,700,680,600,550,400,200,50)
fakeLt=new("lifetable",x=x_example, lx=lx_example, name="fake lifetable")


###################################################
### code chunk number 17: print - show
###################################################
print(fakeLt)


###################################################
### code chunk number 18: head and tail
###################################################
#show head method
head(fakeLt)
#show tail method
tail(fakeLt)


###################################################
### code chunk number 19: create from data frame
###################################################
	#load USA Social Security LT
	data(demoUsa) 
	usaMale07=demoUsa[,c("age", "USSS2007M")]
	usaMale00=demoUsa[,c("age", "USSS2000M")]
	#coerce from data.frame to lifecontingencies requires x and lx names
	names(usaMale07)=c("x","lx")
	names(usaMale00)=c("x","lx")
	#apply coerce methods and changes names
	usaMale07Lt<-as(usaMale07,"lifetable")
	usaMale07Lt@name="USA MALES 2007"
	usaMale00Lt<-as(usaMale00,"lifetable")
	usaMale00Lt@name="USA MALES 2000"
	#create the tables
	##males
	lxIPS55M<-with(demoIta, IPS55M)
	pos2Remove<-which(lxIPS55M %in% c(0,NA))
	lxIPS55M<-lxIPS55M[-pos2Remove]
	xIPS55M<-seq(0,length(lxIPS55M)-1,1)
	##females
	lxIPS55F<-with(demoIta, IPS55F)
	pos2Remove<-which(lxIPS55F %in% c(0,NA))
	lxIPS55F<-lxIPS55F[-pos2Remove]
	xIPS55F<-seq(0,length(lxIPS55F)-1,1)
	#finalize the tables
	ips55M=new("lifetable",x=xIPS55M, lx=lxIPS55M, name="IPS 55 Males")
	ips55F=new("lifetable",x=xIPS55F, lx=lxIPS55F, name="IPS 55 Females")


###################################################
### code chunk number 20: create from survival rates
###################################################
	#use 2002 Italian males life tables
	data(demoIta)
	itaM2002<-demoIta[,c("X","SIM92")]
	names(itaM2002)=c("x","lx")
	itaM2002Lt<-as(itaM2002,"lifetable")
	itaM2002Lt@name="IT 2002 Males"
	#reconvert in data frame
	itaM2002<-as(itaM2002Lt,"data.frame")
	#add qx
	itaM2002$qx<-1-itaM2002$px
	#reduce to 20% one year death probability for ages between 20 and 60
	for(i in 20:60) itaM2002$qx[itaM2002$x==i]=0.2*itaM2002$qx[itaM2002$x==i]
	#otbain the reduced mortality table
	itaM2002reduced<-probs2lifetable(probs=itaM2002[,"qx"], radix=100000,type="qx",name="IT 2002 Males reduced")


###################################################
### code chunk number 21: create a actuarialtable object
###################################################
	#assume 3% interest rate
	fakeAct=new("actuarialtable",x=fakeLt@x, lx=fakeLt@lx, interest=0.03, 
			name="fake actuarialtable")


###################################################
### code chunk number 22: method omega
###################################################
	getOmega(fakeAct)


###################################################
### code chunk number 23: print method
###################################################
#apply method print applied on a life table
print(fakeLt)
#apply method print applied on an actuarial table
print(fakeAct)


###################################################
### code chunk number 24: probability and demographics
###################################################
#using ips55M life table
#probability to survive one year, being at age 20
pxt(ips55M,20,1)
#probability to die within two years, being at age 30
qxt(ips55M,30,2) 
#expected life time between 50 and 70 years
exn(ips55M, 50,20) 


###################################################
### code chunk number 25: fractional ages
###################################################
data(soa08Act) #load Society of Actuaries illustrative life table
pxt(soa08Act,80,0.5,"linear") #linear interpolation (default)
pxt(soa08Act,80,0.5,"constant force") #constant force 
pxt(soa08Act,80,0.5,"hyperbolic") #hyperbolic Balducci's assumption.


###################################################
### code chunk number 26: more than one head
###################################################
pxyt(fakeLt,fakeLt,x=6, y=7, t=2) #joint survival probability
pxyt(fakeLt,fakeLt,x=6, y=7, t=2,status="last") #last survival probability

#evaluate the expected joint life time for a couple aged 65 and 63 using Italina IPS55 tables 
exyt(ips55M, ips55F, x=65,y=63, status="joint")


###################################################
### code chunk number 27: life insurance 1
###################################################
#The APV of a life insurance for a 10-year term life insurance for an
#insured aged 40 @ 4% interest rate is  
Axn(soa08Act, 30,10,i=0.04)
#same as above but payable at the end of month of death
Axn(soa08Act, x=30,n=10,i=0.04,k=12)
#a whole life for a 40 years old insured at @4% is 
Axn(soa08Act, 40) #soa08Act has 6% implicit interest rate
#a 5-year deferred life insurance, 10 years length, 40 years age, @5% interest rate
Axn(soa08Act, x=40,n=10,m=5,i=0.05) 
#Five years annually decreasing term life insurance, age 50.
DAxn(soa08Act, 50,5)
#Increasing 20 years term life insurance, age 40
IAxn(soa08Act, 40,10)


###################################################
### code chunk number 28: pure endowments
###################################################
#evaluate the APV for a n year pure endowment, age x=30, n=35, i=6%
Exn(soa08Act, x=30, n=35, i=0.06)
#try i=3%
Exn(soa08Act, x=30, n=35, i=0.03)


###################################################
### code chunk number 29: annuities
###################################################
#assuming insured's age x=65 and SOA illustrative life table @6% hold for all examples 
#annuity immediate
axn(soa08Act, x=65, m=1)
#annuity due
axn(soa08Act, x=65)
#due with monthly payments of $1000 provision
12*1000*axn(soa08Act, x=65,k=12)
#due with montly payments of $1000 provision, 20 - years term
12*1000*axn(soa08Act, x=65,k=12, n=20)
#immediate with monthly payments of 1000 provision, 20 - years term
12*1000*axn(soa08Act, x=65,k=12,n=20,m=1/12)


###################################################
### code chunk number 30: life insurance 2
###################################################
data(soa08Act) #use SOA MLC exam illustrative life table
#Assume X, aged 30, whishes to buy a 250K 35-years life insurance
#premium paid annually for 15 years @2.5%.
Pa=100000*Axn(soa08Act, x=30,n=35,i=0.025)/axn(soa08Act, x=30,n=15,i=0.025)
Pa
#if premium is paid montly
Pm=100000*Axn(soa08Act, x=30,n=35,i=0.025)/axn(soa08Act, x=30,n=15,i=0.025,k=12)
Pm


###################################################
### code chunk number 31: endowment insurance
###################################################
	#level semiannual premium for an endowment insurance of 10000
	#insured age 50, insurance term is 20 years
	APV=10000*(Axn(soa08Act,50,20)+Exn(soa08Act,50,20))
	P=APV/axn(soa08Act,50,20,k=2)


###################################################
### code chunk number 32: life insurance benefit reserve
###################################################
	P=100000*Axn(soa08Act,x=40,n=20,i=0.03)/axn(soa08Act,x=40,n=20,i=0.03)
	for(t in 0:19) cat("At time ",t," benefit reserve is ", 100000*Axn(soa08Act,x=40+t,n=20-t,i=0.03)-P*axn(soa08Act,x=40+t,n=20-t,i=0.03), "\n")


###################################################
### code chunk number 33: two heads annuity immediate
###################################################
	axn(soa08Act, x=65,m=1)+axn(soa08Act, x=70,m=1)-axyn(soa08Act,soa08Act, x=65,y=70,status="joint",m=1)
	axyn(soa08Act,soa08Act, x=65,y=70, status="last",m=1)


###################################################
### code chunk number 34: revesionary annuity
###################################################
	#assume x aged 65, y aged 60
	axn(soa08Act, x=60,m=1)-axyn(soa08Act,soa08Act, x=65,y=60,status="joint",m=1)


###################################################
### code chunk number 35: rLife1
###################################################
data(soa08Act)
#sample 10 numbers from the Tx distribution
sample1<-rLife(n=10,object=soa08Act,x=0,type="Tx")
#sample 10 numbers from the Kx distribution
sample1<-rLife(n=10,object=soa08Act,x=0,type="Kx")


###################################################
### code chunk number 36: rLife2
###################################################
	#assume an insured aged 29
	#his expected integer number of years until death is 
	exn(soa08Act, x=29,type="curtate")
	#check if we are sampling from a statistically equivalent distribution
	t.test(x=rLife(2000,soa08Act, x=29,type="Kx"),mu=exn(soa08Act, x=29,type="curtate"))$p.value
	#statistically not significant


